@@ -1,9 +1,8 @@
 /*
  *  Exim - an Internet mail transport agent
  *
- *  Copyright (c) The Exim Maintainers 2010 - 2025
- *  Copyright (c) University of Cambridge 1995 - 2009
- *  SPDX-License-Identifier: GPL-2.0-or-later
+ *  Copyright (C) 2017  Exim maintainers
+ *  Copyright (c) University of Cambridge 1995 - 2017
  *
  *  Hash interface functions
  */
@@ -30,130 +29,62 @@ sha1;
 
 /******************************************************************************/
 #ifdef SHA_OPENSSL
-# define HAVE_PARTIAL_SHA
 
 BOOL
 exim_sha_init(hctx * h, hashmethod m)
 {
-# if OPENSSL_VERSION_NUMBER < 0x30000000L
+/*XXX extend for sha512 */
 switch (h->method = m)
   {
   case HASH_SHA1:     h->hashlen = 20; SHA1_Init  (&h->u.sha1);     break;
   case HASH_SHA2_256: h->hashlen = 32; SHA256_Init(&h->u.sha2_256); break;
   case HASH_SHA2_384: h->hashlen = 48; SHA384_Init(&h->u.sha2_512); break;
   case HASH_SHA2_512: h->hashlen = 64; SHA512_Init(&h->u.sha2_512); break;
-#  ifdef EXIM_HAVE_SHA3
-  case HASH_SHA3_224: h->hashlen = 28;
-		      EVP_DigestInit(h->u.mctx = EVP_MD_CTX_new(), EVP_sha3_224());
-		      break;
-  case HASH_SHA3_256: h->hashlen = 32;
-		      EVP_DigestInit(h->u.mctx = EVP_MD_CTX_new(), EVP_sha3_256());
-		      break;
-  case HASH_SHA3_384: h->hashlen = 48;
-		      EVP_DigestInit(h->u.mctx = EVP_MD_CTX_new(), EVP_sha3_384());
-		      break;
-  case HASH_SHA3_512: h->hashlen = 64;
-		      EVP_DigestInit(h->u.mctx = EVP_MD_CTX_new(), EVP_sha3_512());
-		      break;
-#  endif
   default:	      h->hashlen = 0; return FALSE;
   }
 return TRUE;
-
-# else
-EVP_MD * md;
-
-h->hashlen = 0;
-if (!(h->u.mctx = EVP_MD_CTX_new())) return FALSE;
-switch (h->method = m)
-  {
-  case HASH_SHA1:     h->hashlen = 20; md = EVP_MD_fetch(NULL, "SHA1", NULL); break;
-  case HASH_SHA2_256: h->hashlen = 32; md = EVP_MD_fetch(NULL, "SHA2-256", NULL); break;
-  case HASH_SHA2_384: h->hashlen = 48; md = EVP_MD_fetch(NULL, "SHA2-384", NULL); break;
-  case HASH_SHA2_512: h->hashlen = 64; md = EVP_MD_fetch(NULL, "SHA2-512", NULL); break;
-  case HASH_SHA3_224: h->hashlen = 28; md = EVP_MD_fetch(NULL, "SHA3-224", NULL); break;
-  case HASH_SHA3_256: h->hashlen = 32; md = EVP_MD_fetch(NULL, "SHA3-256", NULL); break;
-  case HASH_SHA3_384: h->hashlen = 48; md = EVP_MD_fetch(NULL, "SHA3-384", NULL); break;
-  case HASH_SHA3_512: h->hashlen = 64; md = EVP_MD_fetch(NULL, "SHA3-512", NULL); break;
-  default:	      return FALSE;
-  }
-if (md && EVP_DigestInit_ex(h->u.mctx, md, NULL))
-  return TRUE;
-
-h->hashlen = 0;
-return FALSE;
-# endif
 }
 
 
 void
 exim_sha_update(hctx * h, const uschar * data, int len)
 {
-# if OPENSSL_VERSION_NUMBER < 0x30000000L
 switch (h->method)
   {
   case HASH_SHA1:     SHA1_Update  (&h->u.sha1,     data, len); break;
   case HASH_SHA2_256: SHA256_Update(&h->u.sha2_256, data, len); break;
   case HASH_SHA2_384: SHA384_Update(&h->u.sha2_512, data, len); break;
   case HASH_SHA2_512: SHA512_Update(&h->u.sha2_512, data, len); break;
-#  ifdef EXIM_HAVE_SHA3
-  case HASH_SHA3_224:
-  case HASH_SHA3_256:
-  case HASH_SHA3_384:
-  case HASH_SHA3_512: EVP_DigestUpdate(h->u.mctx, data, len); break;
-#  endif
   /* should be blocked by init not handling these, but be explicit to
   guard against accidents later (and hush up clang -Wswitch) */
   default: assert(0);
   }
-
-# else
-
-EVP_DigestUpdate(h->u.mctx, data, len);
-# endif
 }
 
 
 void
 exim_sha_finish(hctx * h, blob * b)
 {
-/* Hashing is sufficient to purify any tainted input */
-b->data = store_get(b->len = h->hashlen, GET_UNTAINTED);
-
-# if OPENSSL_VERSION_NUMBER < 0x30000000L
+b->data = store_get(b->len = h->hashlen);
 switch (h->method)
   {
   case HASH_SHA1:     SHA1_Final  (b->data, &h->u.sha1);     break;
   case HASH_SHA2_256: SHA256_Final(b->data, &h->u.sha2_256); break;
   case HASH_SHA2_384: SHA384_Final(b->data, &h->u.sha2_512); break;
   case HASH_SHA2_512: SHA512_Final(b->data, &h->u.sha2_512); break;
-#  ifdef EXIM_HAVE_SHA3
-  case HASH_SHA3_224:
-  case HASH_SHA3_256:
-  case HASH_SHA3_384:
-  case HASH_SHA3_512: EVP_DigestFinal(h->u.mctx, b->data, NULL); break;
-#  endif
   default: assert(0);
   }
-
-# else
-
-EVP_DigestFinal_ex(h->u.mctx, b->data, NULL);
-EVP_MD_free((EVP_MD *) EVP_MD_CTX_get0_md(h->u.mctx));
-EVP_MD_CTX_free(h->u.mctx);
-
-# endif
 }
 
 
 
 #elif defined(SHA_GNUTLS)
-# define HAVE_PARTIAL_SHA
 /******************************************************************************/
 
 BOOL
 exim_sha_init(hctx * h, hashmethod m)
 {
+/*XXX extend for sha512 */
 switch (h->method = m)
   {
   case HASH_SHA1:     h->hashlen = 20; gnutls_hash_init(&h->sha, GNUTLS_DIG_SHA1);   break;
@@ -161,7 +92,6 @@ switch (h->method = m)
   case HASH_SHA2_384: h->hashlen = 48; gnutls_hash_init(&h->sha, GNUTLS_DIG_SHA384); break;
   case HASH_SHA2_512: h->hashlen = 64; gnutls_hash_init(&h->sha, GNUTLS_DIG_SHA512); break;
 #ifdef EXIM_HAVE_SHA3
-  case HASH_SHA3_224: h->hashlen = 28; gnutls_hash_init(&h->sha, GNUTLS_DIG_SHA3_224); break;
   case HASH_SHA3_256: h->hashlen = 32; gnutls_hash_init(&h->sha, GNUTLS_DIG_SHA3_256); break;
   case HASH_SHA3_384: h->hashlen = 48; gnutls_hash_init(&h->sha, GNUTLS_DIG_SHA3_384); break;
   case HASH_SHA3_512: h->hashlen = 64; gnutls_hash_init(&h->sha, GNUTLS_DIG_SHA3_512); break;
@@ -182,19 +112,19 @@ gnutls_hash(h->sha, data, len);
 void
 exim_sha_finish(hctx * h, blob * b)
 {
-b->data = store_get(b->len = h->hashlen, GET_UNTAINTED);
+b->data = store_get(b->len = h->hashlen);
 gnutls_hash_output(h->sha, b->data);
 }
 
 
 
 #elif defined(SHA_GCRYPT)
-# define HAVE_PARTIAL_SHA
 /******************************************************************************/
 
 BOOL
 exim_sha_init(hctx * h, hashmethod m)
 {
+/*XXX extend for sha512 */
 switch (h->method = m)
   {
   case HASH_SHA1:     h->hashlen = 20; gcry_md_open(&h->sha, GCRY_MD_SHA1, 0);   break;
@@ -220,13 +150,55 @@ gcry_md_write(h->sha, data, len);
 void
 exim_sha_finish(hctx * h, blob * b)
 {
-b->data = store_get(b->len = h->hashlen, GET_UNTAINTED);
+b->data = store_get(b->len = h->hashlen);
 memcpy(b->data, gcry_md_read(h->sha, 0), h->hashlen);
 }
 
 
 
 
+#elif defined(SHA_POLARSSL)
+/******************************************************************************/
+
+BOOL
+exim_sha_init(hctx * h, hashmethod m)
+{
+/*XXX extend for sha512 */
+switch (h->method = m)
+  {
+  case HASH_SHA1:   h->hashlen = 20; sha1_starts(&h->u.sha1);    break;
+  case HASH_SHA2_256: h->hashlen = 32; sha2_starts(&h->u.sha2, 0); break;
+  default:	    h->hashlen = 0; return FALSE;
+  }
+return TRUE;
+}
+
+
+void
+exim_sha_update(hctx * h, const uschar * data, int len)
+{
+switch (h->method)
+  {
+  case HASH_SHA1:   sha1_update(h->u.sha1, US data, len); break;
+  case HASH_SHA2_256: sha2_update(h->u.sha2, US data, len); break;
+  }
+}
+
+
+void
+exim_sha_finish(hctx * h, blob * b)
+{
+b->data = store_get(b->len = h->hashlen);
+switch (h->method)
+  {
+  case HASH_SHA1:   sha1_finish(h->u.sha1, b->data); break;
+  case HASH_SHA2_256: sha2_finish(h->u.sha2, b->data); break;
+  }
+}
+
+
+
+
 #elif defined(SHA_NATIVE)
 /******************************************************************************/
 /* Only sha-1 supported */
@@ -269,18 +241,19 @@ Returns:     nothing
 static void
 native_sha1_mid(sha1 *base, const uschar *text)
 {
+int i;
 uint A, B, C, D, E;
 uint W[80];
 
 base->length += 64;
 
-for (int i = 0; i < 16; i++)
+for (i = 0; i < 16; i++)
   {
   W[i] = ((uint)text[0] << 24) | (text[1] << 16) | (text[2] << 8) | text[3];
   text += 4;
   }
 
-for (int i = 16; i < 80; i++)
+for (i = 16; i < 80; i++)
   {
   register unsigned int x = W[i-3] ^ W[i-8] ^ W[i-14] ^ W[i-16];
   W[i] = (x << 1) | (x >> 31);
@@ -292,7 +265,7 @@ C = base->H[2];
 D = base->H[3];
 E = base->H[4];
 
-for (int i = 0; i < 20; i++)
+for (i = 0; i < 20; i++)
   {
   unsigned int T;
   T = ((A << 5) | (A >> 27)) + ((B & C) | ((~B) & D)) + E + W[i] + 0x5a827999;
@@ -303,7 +276,7 @@ for (int i = 0; i < 20; i++)
   A = T;
   }
 
-for (int i = 20; i < 40; i++)
+for (i = 20; i < 40; i++)
   {
   unsigned int T;
   T = ((A << 5) | (A >> 27)) + (B ^ C ^ D) + E + W[i] + 0x6ed9eba1;
@@ -314,7 +287,7 @@ for (int i = 20; i < 40; i++)
   A = T;
   }
 
-for (int i = 40; i < 60; i++)
+for (i = 40; i < 60; i++)
   {
   unsigned int T;
   T = ((A << 5) | (A >> 27)) + ((B & C) | (B & D) | (C & D)) + E + W[i] +
@@ -326,7 +299,7 @@ for (int i = 40; i < 60; i++)
   A = T;
   }
 
-for (int i = 60; i < 80; i++)
+for (i = 60; i < 80; i++)
   {
   unsigned int T;
   T = ((A << 5) | (A >> 27)) + (B ^ C ^ D) + E + W[i] + 0xca62c1d6;
@@ -365,8 +338,9 @@ Returns:    nothing
 */
 
 static void
-native_sha1_end(sha1 * base, const uschar * text, int length, uschar * digest)
+native_sha1_end(sha1 *base, const uschar *text, int length, uschar *digest)
 {
+int i;
 uschar work[64];
 
 /* Process in chunks of 64 until we have less than 64 bytes left. */
@@ -383,7 +357,7 @@ out to 64, process it, and then set up the final chunk as 56 bytes of
 padding. If it has less than 56 bytes, we pad it out to 56 bytes as the
 final chunk. */
 
-if (length) memcpy(work, text, length);
+memcpy(work, text, length);
 work[length] = 0x80;
 
 if (length > 55)
@@ -394,7 +368,9 @@ if (length > 55)
   memset(work, 0, 56);
   }
 else
+  {
   memset(work+length+1, 0, 55-length);
+  }
 
 /* The final 8 bytes of the final chunk are a 64-bit representation of the
 length of the input string *bits*, before padding, high order word first, and
@@ -417,7 +393,7 @@ native_sha1_mid(base, work);
 
 /* Pass back the result, high-order byte first in each word. */
 
-for (int i = 0; i < 5; i++)
+for (i = 0; i < 5; i++)
   {
   register int x = base->H[i];
   *digest++ = (x >> 24) & 0xff;
@@ -452,7 +428,7 @@ native_sha1_mid(&h->sha1, US data);	/* implicit size always 64 */
 void
 exim_sha_finish(hctx * h, blob * b)
 {
-b->data = store_get(b->len = h->hashlen, GET_UNTAINTED);
+b->data = store_get(b->len = h->hashlen);
 
 native_sha1_end(&h->sha1, NULL, 0, b->data);
 }
@@ -516,14 +492,6 @@ memcpy(digest, b.data, 20);
 
 
 
-#ifdef HAVE_PARTIAL_SHA
-# undef HAVE_PARTIAL_SHA
-void
-exim_sha_update_string(hctx * h, const uschar * s)
-{
-if (s) exim_sha_update(h, s, Ustrlen(s));
-}
-#endif
 
 
 
@@ -533,7 +501,7 @@ if (s) exim_sha_update(h, s, Ustrlen(s));
 **************************************************
 *************************************************/
 
-#ifdef STAND_ALONE
+# ifdef STAND_ALONE
 
 /* Test values. The first 128 may contain binary zeros and have increasing
 length. */
@@ -848,6 +816,6 @@ printf("Computed:  %s\n", s);
 if (strcmp(s, atest) != 0) printf("*** No match ***\n");
 
 }
-#endif	/*STAND_ALONE*/
+# endif	/*STAND_ALONE*/
 
 /* End of File */
